@@ -2,8 +2,8 @@
  *
  * SEHANDLERS.C - Service and host event and state handlers for Nagios
  *
- * Copyright (c) 1999-2004 Ethan Galstad (nagios@nagios.org)
- * Last Modified:   12-19-2004
+ * Copyright (c) 1999-2007 Ethan Galstad (nagios@nagios.org)
+ * Last Modified:   11-10-2007
  *
  * License:
  *
@@ -23,9 +23,11 @@
  *****************************************************************************/
 
 #include "../include/config.h"
+#include "../include/comments.h"
 #include "../include/common.h"
 #include "../include/statusdata.h"
 #include "../include/downtime.h"
+#include "../include/macros.h"
 #include "../include/nagios.h"
 #include "../include/perfdata.h"
 #include "../include/broker.h"
@@ -38,6 +40,9 @@ extern int             obsess_over_hosts;
 extern int             log_event_handlers;
 extern int             log_host_retries;
 
+extern unsigned long   next_event_id;
+extern unsigned long   next_problem_id;
+
 extern int             event_handler_timeout;
 extern int             ocsp_timeout;
 extern int             ochp_timeout;
@@ -46,9 +51,13 @@ extern char            *macro_x[MACRO_X_COUNT];
 
 extern char            *global_host_event_handler;
 extern char            *global_service_event_handler;
+extern command         *global_host_event_handler_ptr;
+extern command         *global_service_event_handler_ptr;
 
 extern char            *ocsp_command;
 extern char            *ochp_command;
+extern command         *ocsp_command_ptr;
+extern command         *ochp_command_ptr;
 
 extern time_t          program_start;
 
@@ -61,17 +70,17 @@ extern time_t          program_start;
 
 /* handles service check results in an obsessive compulsive manner... */
 int obsessive_compulsive_service_check_processor(service *svc){
-	char raw_command_line[MAX_COMMAND_BUFFER];
-	char processed_command_line[MAX_COMMAND_BUFFER];
-	char temp_buffer[MAX_INPUT_BUFFER];
-	host *temp_host;
+	char *raw_command=NULL;
+	char *processed_command=NULL;
+	host *temp_host=NULL;
 	int early_timeout=FALSE;
-	double exectime;
+	double exectime=0.0;
 	int macro_options=STRIP_ILLEGAL_MACRO_CHARS|ESCAPE_MACRO_CHARS;
 
-#ifdef DEBUG0
-	printf("obsessive_compulsive_service_check_processor() start\n");
-#endif
+	log_debug_info(DEBUGL_FUNCTIONS,0,"obsessive_compulsive_service_check_processor()\n");
+
+	if(svc==NULL)
+		return ERROR;
 
 	/* bail out if we shouldn't be obsessing */
 	if(obsess_over_services==FALSE)
@@ -84,43 +93,39 @@ int obsessive_compulsive_service_check_processor(service *svc){
 		return ERROR;
 
 	/* find the associated host */
-	temp_host=find_host(svc->host_name);
+	if((temp_host=(host *)svc->host_ptr)==NULL)
+		return ERROR;
 
 	/* update service macros */
 	clear_volatile_macros();
 	grab_host_macros(temp_host);
 	grab_service_macros(svc);
-	grab_summary_macros(NULL);
 
 	/* get the raw command line */
-	get_raw_command_line(ocsp_command,raw_command_line,sizeof(raw_command_line),macro_options);
-	strip(raw_command_line);
+	get_raw_command_line(ocsp_command_ptr,ocsp_command,&raw_command,macro_options);
+	if(raw_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tRaw obsessive compulsive service processor command line: %s\n",raw_command_line);
-#endif
+	log_debug_info(DEBUGL_CHECKS,2,"Raw obsessive compulsive service processor command line: %s\n",raw_command);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line),macro_options);
+	process_macros(raw_command,&processed_command,macro_options);
+	if(processed_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tProcessed obsessive compulsive service processor command line: %s\n",processed_command_line);
-#endif
+	log_debug_info(DEBUGL_CHECKS,2,"Processed obsessive compulsive service processor command line: %s\n",processed_command);
 
 	/* run the command */
-	my_system(processed_command_line,ocsp_timeout,&early_timeout,&exectime,NULL,0);
+	my_system(processed_command,ocsp_timeout,&early_timeout,&exectime,NULL,0);
 
 	/* check to see if the command timed out */
-	if(early_timeout==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"Warning: OCSP command '%s' for service '%s' on host '%s' timed out after %d seconds\n",processed_command_line,svc->description,svc->host_name,ocsp_timeout);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_logs_and_console(temp_buffer,NSLOG_RUNTIME_WARNING,TRUE);
-	        }
-	
-#ifdef DEBUG0
-	printf("obsessive_compulsive_service_check_processor() end\n");
-#endif
+	if(early_timeout==TRUE)
+		logit(NSLOG_RUNTIME_WARNING,TRUE,"Warning: OCSP command '%s' for service '%s' on host '%s' timed out after %d seconds\n",processed_command,svc->description,svc->host_name,ocsp_timeout);
 
+	/* free memory */
+	my_free(raw_command);
+	my_free(processed_command);
+	
 	return OK;
         }
 
@@ -128,16 +133,16 @@ int obsessive_compulsive_service_check_processor(service *svc){
 
 /* handles host check results in an obsessive compulsive manner... */
 int obsessive_compulsive_host_check_processor(host *hst){
-	char raw_command_line[MAX_COMMAND_BUFFER];
-	char processed_command_line[MAX_COMMAND_BUFFER];
-	char temp_buffer[MAX_INPUT_BUFFER];
+	char *raw_command=NULL;
+	char *processed_command=NULL;
 	int early_timeout=FALSE;
-	double exectime;
+	double exectime=0.0;
 	int macro_options=STRIP_ILLEGAL_MACRO_CHARS|ESCAPE_MACRO_CHARS;
 
-#ifdef DEBUG0
-	printf("obsessive_compulsive_host_check_processor() start\n");
-#endif
+	log_debug_info(DEBUGL_FUNCTIONS,0,"obsessive_compulsive_host_check_processor()\n");
+
+	if(hst==NULL)
+		return ERROR;
 
 	/* bail out if we shouldn't be obsessing */
 	if(obsess_over_hosts==FALSE)
@@ -152,36 +157,31 @@ int obsessive_compulsive_host_check_processor(host *hst){
 	/* update macros */
 	clear_volatile_macros();
 	grab_host_macros(hst);
-	grab_summary_macros(NULL);
 
 	/* get the raw command line */
-	get_raw_command_line(ochp_command,raw_command_line,sizeof(raw_command_line),macro_options);
-	strip(raw_command_line);
+	get_raw_command_line(ochp_command_ptr,ochp_command,&raw_command,macro_options);
+	if(raw_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tRaw obsessive compulsive host processor command line: %s\n",raw_command_line);
-#endif
+	log_debug_info(DEBUGL_CHECKS,2,"Raw obsessive compulsive host processor command line: %s\n",raw_command);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line),macro_options);
+	process_macros(raw_command,&processed_command,macro_options);
+	if(processed_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tProcessed obsessive compulsive host processor command line: %s\n",processed_command_line);
-#endif
+	log_debug_info(DEBUGL_CHECKS,2,"Processed obsessive compulsive host processor command line: %s\n",processed_command);
 
 	/* run the command */
-	my_system(processed_command_line,ochp_timeout,&early_timeout,&exectime,NULL,0);
+	my_system(processed_command,ochp_timeout,&early_timeout,&exectime,NULL,0);
 
 	/* check to see if the command timed out */
-	if(early_timeout==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"Warning: OCHP command '%s' for host '%s' timed out after %d seconds\n",processed_command_line,hst->name,ochp_timeout);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_logs_and_console(temp_buffer,NSLOG_RUNTIME_WARNING,TRUE);
-	        }
+	if(early_timeout==TRUE)
+		logit(NSLOG_RUNTIME_WARNING,TRUE,"Warning: OCHP command '%s' for host '%s' timed out after %d seconds\n",processed_command,hst->name,ochp_timeout);
 
-#ifdef DEBUG0
-	printf("obsessive_compulsive_host_check_processor() end\n");
-#endif
+	/* free memory */
+	my_free(raw_command);
+	my_free(processed_command);
 
 	return OK;
         }
@@ -196,11 +196,12 @@ int obsessive_compulsive_host_check_processor(host *hst){
 
 /* handles changes in the state of a service */
 int handle_service_event(service *svc){
-	host *temp_host;
+	host *temp_host=NULL;
 
-#ifdef DEBUG0
-	printf("handle_service_event() start\n");
-#endif
+	log_debug_info(DEBUGL_FUNCTIONS,0,"handle_service_event()\n");
+
+	if(svc==NULL)
+		return ERROR;
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
@@ -214,13 +215,13 @@ int handle_service_event(service *svc){
 		return OK;
 
 	/* find the host */
-	temp_host=find_host(svc->host_name);
+	if((temp_host=(host *)svc->host_ptr)==NULL)
+		return ERROR;
 
 	/* update service macros */
 	clear_volatile_macros();
 	grab_host_macros(temp_host);
 	grab_service_macros(svc);
-	grab_summary_macros(NULL);
 
 	/* run the global service event handler */
 	run_global_service_event_handler(svc);
@@ -232,10 +233,6 @@ int handle_service_event(service *svc){
 	/* check for external commands - the event handler may have given us some directives... */
 	check_for_external_commands();
 
-#ifdef DEBUG0
-	printf("handle_service_event() end\n");
-#endif
-
 	return OK;
         }
 
@@ -243,20 +240,23 @@ int handle_service_event(service *svc){
 
 /* runs the global service event handler */
 int run_global_service_event_handler(service *svc){
-	char raw_command_line[MAX_COMMAND_BUFFER];
-	char processed_command_line[MAX_COMMAND_BUFFER];
-	char command_output[MAX_INPUT_BUFFER];
-	char temp_buffer[MAX_INPUT_BUFFER];
+	char *raw_command=NULL;
+	char *processed_command=NULL;
+	char *command_output=NULL;
 	int early_timeout=FALSE;
 	double exectime=0.0;
 	int result=0;
+#ifdef USE_EVENT_BROKER
 	struct timeval start_time;
 	struct timeval end_time;
+#endif
 	int macro_options=STRIP_ILLEGAL_MACRO_CHARS|ESCAPE_MACRO_CHARS;
 
-#ifdef DEBUG0
-	printf("run_global_service_event_handler() start\n");
-#endif
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"run_global_service_event_handler()\n");
+
+	if(svc==NULL)
+		return ERROR;
 
 	/* bail out if we shouldn't be running event handlers */
 	if(enable_event_handlers==FALSE)
@@ -266,8 +266,12 @@ int run_global_service_event_handler(service *svc){
 	if(global_service_event_handler==NULL)
 		return ERROR;
 
+	log_debug_info(DEBUGL_EVENTHANDLERS,1,"Running global event handler for service '%s' on host '%s'...\n",svc->description,svc->host_name);
+
+#ifdef USE_EVENT_BROKER
 	/* get start time */
 	gettimeofday(&start_time,NULL);
+#endif
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
@@ -277,47 +281,43 @@ int run_global_service_event_handler(service *svc){
 #endif
 
 	/* get the raw command line */
-	get_raw_command_line(global_service_event_handler,raw_command_line,sizeof(raw_command_line),macro_options);
-	strip(raw_command_line);
+	get_raw_command_line(global_service_event_handler_ptr,global_service_event_handler,&raw_command,macro_options);
+	if(raw_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tRaw global service event handler command line: %s\n",raw_command_line);
-#endif
+	log_debug_info(DEBUGL_EVENTHANDLERS,2,"Raw global service event handler command line: %s\n",raw_command);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line),macro_options);
+	process_macros(raw_command,&processed_command,macro_options);
+	if(processed_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tProcessed global service event handler command line: %s\n",processed_command_line);
-#endif
+	log_debug_info(DEBUGL_EVENTHANDLERS,2,"Processed global service event handler command line: %s\n",processed_command);
 
-	if(log_event_handlers==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"GLOBAL SERVICE EVENT HANDLER: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,macro_x[MACRO_SERVICESTATE],macro_x[MACRO_SERVICESTATETYPE],macro_x[MACRO_SERVICEATTEMPT],global_service_event_handler);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_all_logs(temp_buffer,NSLOG_EVENT_HANDLER);
-	        }
+	if(log_event_handlers==TRUE)
+		logit(NSLOG_EVENT_HANDLER,FALSE,"GLOBAL SERVICE EVENT HANDLER: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,macro_x[MACRO_SERVICESTATE],macro_x[MACRO_SERVICESTATETYPE],macro_x[MACRO_SERVICEATTEMPT],global_service_event_handler);
 
 	/* run the command */
-	result=my_system(processed_command_line,event_handler_timeout,&early_timeout,&exectime,command_output,sizeof(command_output)-1);
+	result=my_system(processed_command,event_handler_timeout,&early_timeout,&exectime,&command_output,0);
 
 	/* check to see if the event handler timed out */
-	if(early_timeout==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Global service event handler command '%s' timed out after %d seconds\n",processed_command_line,event_handler_timeout);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_logs_and_console(temp_buffer,NSLOG_EVENT_HANDLER | NSLOG_RUNTIME_WARNING,TRUE);
-	        }
+	if(early_timeout==TRUE)
+		logit(NSLOG_EVENT_HANDLER | NSLOG_RUNTIME_WARNING,TRUE,"Warning: Global service event handler command '%s' timed out after %d seconds\n",processed_command,event_handler_timeout);
 
+#ifdef USE_EVENT_BROKER
 	/* get end time */
 	gettimeofday(&end_time,NULL);
+#endif
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
-	broker_event_handler(NEBTYPE_EVENTHANDLER_END,NEBFLAG_NONE,NEBATTR_NONE,GLOBAL_SERVICE_EVENTHANDLER,(void *)svc,svc->current_state,svc->state_type,start_time,end_time,exectime,event_handler_timeout,early_timeout,result,global_service_event_handler,processed_command_line,command_output,NULL);
+	broker_event_handler(NEBTYPE_EVENTHANDLER_END,NEBFLAG_NONE,NEBATTR_NONE,GLOBAL_SERVICE_EVENTHANDLER,(void *)svc,svc->current_state,svc->state_type,start_time,end_time,exectime,event_handler_timeout,early_timeout,result,global_service_event_handler,processed_command,command_output,NULL);
 #endif
 
-#ifdef DEBUG0
-	printf("run_global_service_event_handler() end\n");
-#endif
+	/* free memory */
+	my_free(command_output);
+	my_free(raw_command);
+	my_free(processed_command);
 
 	return OK;
         }
@@ -326,27 +326,34 @@ int run_global_service_event_handler(service *svc){
 
 /* runs a service event handler command */
 int run_service_event_handler(service *svc){
-	char raw_command_line[MAX_COMMAND_BUFFER];
-	char processed_command_line[MAX_COMMAND_BUFFER];
-	char command_output[MAX_INPUT_BUFFER];
-	char temp_buffer[MAX_INPUT_BUFFER];
+	char *raw_command=NULL;
+	char *processed_command=NULL;
+	char *command_output=NULL;
 	int early_timeout=FALSE;
 	double exectime=0.0;
 	int result=0;
+#ifdef USE_EVENT_BROKER
 	struct timeval start_time;
 	struct timeval end_time;
+#endif
 	int macro_options=STRIP_ILLEGAL_MACRO_CHARS|ESCAPE_MACRO_CHARS;
 
-#ifdef DEBUG0
-	printf("run_service_event_handler() start\n");
-#endif
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"run_service_event_handler()\n");
+
+	if(svc==NULL)
+		return ERROR;
 
 	/* bail if there's no command */
 	if(svc->event_handler==NULL)
 		return ERROR;
 
+	log_debug_info(DEBUGL_EVENTHANDLERS,1,"Running event handler for service '%s' on host '%s'...\n",svc->description,svc->host_name);
+
+#ifdef USE_EVENT_BROKER
 	/* get start time */
 	gettimeofday(&start_time,NULL);
+#endif
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
@@ -356,47 +363,43 @@ int run_service_event_handler(service *svc){
 #endif
 
 	/* get the raw command line */
-	get_raw_command_line(svc->event_handler,raw_command_line,sizeof(raw_command_line),macro_options);
-	strip(raw_command_line);
+	get_raw_command_line(svc->event_handler_ptr,svc->event_handler,&raw_command,macro_options);
+	if(raw_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tRaw service event handler command line: %s\n",raw_command_line);
-#endif
+	log_debug_info(DEBUGL_EVENTHANDLERS,2,"Raw service event handler command line: %s\n",raw_command);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line),macro_options);
+	process_macros(raw_command,&processed_command,macro_options);
+	if(processed_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tProcessed service event handler command line: %s\n",processed_command_line);
-#endif
+	log_debug_info(DEBUGL_EVENTHANDLERS,2,"Processed service event handler command line: %s\n",processed_command);
 
-	if(log_event_handlers==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"SERVICE EVENT HANDLER: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,macro_x[MACRO_SERVICESTATE],macro_x[MACRO_SERVICESTATETYPE],macro_x[MACRO_SERVICEATTEMPT],svc->event_handler);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_all_logs(temp_buffer,NSLOG_EVENT_HANDLER);
-	        }
+	if(log_event_handlers==TRUE)
+		logit(NSLOG_EVENT_HANDLER,FALSE,"SERVICE EVENT HANDLER: %s;%s;%s;%s;%s;%s\n",svc->host_name,svc->description,macro_x[MACRO_SERVICESTATE],macro_x[MACRO_SERVICESTATETYPE],macro_x[MACRO_SERVICEATTEMPT],svc->event_handler);
 
 	/* run the command */
-	result=my_system(processed_command_line,event_handler_timeout,&early_timeout,&exectime,command_output,sizeof(command_output)-1);
+	result=my_system(processed_command,event_handler_timeout,&early_timeout,&exectime,&command_output,0);
 
 	/* check to see if the event handler timed out */
-	if(early_timeout==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Service event handler command '%s' timed out after %d seconds\n",processed_command_line,event_handler_timeout);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_logs_and_console(temp_buffer,NSLOG_EVENT_HANDLER | NSLOG_RUNTIME_WARNING,TRUE);
-	        }
+	if(early_timeout==TRUE)
+		logit(NSLOG_EVENT_HANDLER | NSLOG_RUNTIME_WARNING,TRUE,"Warning: Service event handler command '%s' timed out after %d seconds\n",processed_command,event_handler_timeout);
 
+#ifdef USE_EVENT_BROKER
 	/* get end time */
 	gettimeofday(&end_time,NULL);
+#endif
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
-	broker_event_handler(NEBTYPE_EVENTHANDLER_END,NEBFLAG_NONE,NEBATTR_NONE,SERVICE_EVENTHANDLER,(void *)svc,svc->current_state,svc->state_type,start_time,end_time,exectime,event_handler_timeout,early_timeout,result,svc->event_handler,processed_command_line,command_output,NULL);
+	broker_event_handler(NEBTYPE_EVENTHANDLER_END,NEBFLAG_NONE,NEBATTR_NONE,SERVICE_EVENTHANDLER,(void *)svc,svc->current_state,svc->state_type,start_time,end_time,exectime,event_handler_timeout,early_timeout,result,svc->event_handler,processed_command,command_output,NULL);
 #endif
 
-#ifdef DEBUG0
-	printf("run_service_event_handler() end\n");
-#endif
+	/* free memory */
+	my_free(command_output);
+	my_free(raw_command);
+	my_free(processed_command);
 
 	return OK;
         }
@@ -412,9 +415,10 @@ int run_service_event_handler(service *svc){
 /* handles a change in the status of a host */
 int handle_host_event(host *hst){
 
-#ifdef DEBUG0
-	printf("handle_host_event() start\n");
-#endif
+	log_debug_info(DEBUGL_FUNCTIONS,0,"handle_host_event()\n");
+
+	if(hst==NULL)
+		return ERROR;
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
@@ -430,7 +434,6 @@ int handle_host_event(host *hst){
 	/* update host macros */
 	clear_volatile_macros();
 	grab_host_macros(hst);
-	grab_summary_macros(NULL);
 
 	/* run the global host event handler */
 	run_global_host_event_handler(hst);
@@ -442,30 +445,29 @@ int handle_host_event(host *hst){
 	/* check for external commands - the event handler may have given us some directives... */
 	check_for_external_commands();
 
-#ifdef DEBUG0
-	printf("handle_host_event() end\n");
-#endif
-
 	return OK;
         }
 
 
 /* runs the global host event handler */
 int run_global_host_event_handler(host *hst){
-	char raw_command_line[MAX_COMMAND_BUFFER];
-	char processed_command_line[MAX_COMMAND_BUFFER];
-	char command_output[MAX_INPUT_BUFFER];
-	char temp_buffer[MAX_INPUT_BUFFER];
+	char *raw_command=NULL;
+	char *processed_command=NULL;
+	char *command_output=NULL;
 	int early_timeout=FALSE;
 	double exectime=0.0;
 	int result=0;
+#ifdef USE_EVENT_BROKER
 	struct timeval start_time;
 	struct timeval end_time;
+#endif
 	int macro_options=STRIP_ILLEGAL_MACRO_CHARS|ESCAPE_MACRO_CHARS;
 
-#ifdef DEBUG0
-	printf("run_global_host_event_handler() start\n");
-#endif
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"run_global_host_event_handler()\n");
+
+	if(hst==NULL)
+		return ERROR;
 
 	/* bail out if we shouldn't be running event handlers */
 	if(enable_event_handlers==FALSE)
@@ -475,8 +477,12 @@ int run_global_host_event_handler(host *hst){
 	if(global_host_event_handler==NULL)
 		return ERROR;
 
+	log_debug_info(DEBUGL_EVENTHANDLERS,1,"Running global event handler for host '%s'..\n",hst->name);
+
+#ifdef USE_EVENT_BROKER
 	/* get start time */
 	gettimeofday(&start_time,NULL);
+#endif
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
@@ -486,47 +492,43 @@ int run_global_host_event_handler(host *hst){
 #endif
 
 	/* get the raw command line */
-	get_raw_command_line(global_host_event_handler,raw_command_line,sizeof(raw_command_line),macro_options);
-	strip(raw_command_line);
+	get_raw_command_line(global_host_event_handler_ptr,global_host_event_handler,&raw_command,macro_options);
+	if(raw_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tRaw global host event handler command line: %s\n",raw_command_line);
-#endif
+	log_debug_info(DEBUGL_EVENTHANDLERS,2,"Raw global host event handler command line: %s\n",raw_command);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line),macro_options);
+	process_macros(raw_command,&processed_command,macro_options);
+	if(processed_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tProcessed global host event handler command line: %s\n",processed_command_line);
-#endif
+	log_debug_info(DEBUGL_EVENTHANDLERS,2,"Processed global host event handler command line: %s\n",processed_command);
 
-	if(log_event_handlers==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"GLOBAL HOST EVENT HANDLER: %s;%s;%s;%s;%s\n",hst->name,macro_x[MACRO_HOSTSTATE],macro_x[MACRO_HOSTSTATETYPE],macro_x[MACRO_HOSTATTEMPT],global_host_event_handler);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_all_logs(temp_buffer,NSLOG_EVENT_HANDLER);
-	        }
+	if(log_event_handlers==TRUE)
+		logit(NSLOG_EVENT_HANDLER,FALSE,"GLOBAL HOST EVENT HANDLER: %s;%s;%s;%s;%s\n",hst->name,macro_x[MACRO_HOSTSTATE],macro_x[MACRO_HOSTSTATETYPE],macro_x[MACRO_HOSTATTEMPT],global_host_event_handler);
 
 	/* run the command */
-	result=my_system(processed_command_line,event_handler_timeout,&early_timeout,&exectime,command_output,sizeof(command_output)-1);
+	result=my_system(processed_command,event_handler_timeout,&early_timeout,&exectime,&command_output,0);
 
 	/* check for a timeout in the execution of the event handler command */
-	if(early_timeout==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Global host event handler command '%s' timed out after %d seconds\n",processed_command_line,event_handler_timeout);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_logs_and_console(temp_buffer,NSLOG_EVENT_HANDLER | NSLOG_RUNTIME_WARNING,TRUE);
-	        }
+	if(early_timeout==TRUE)
+		logit(NSLOG_EVENT_HANDLER | NSLOG_RUNTIME_WARNING,TRUE,"Warning: Global host event handler command '%s' timed out after %d seconds\n",processed_command,event_handler_timeout);
 
+#ifdef USE_EVENT_BROKER
 	/* get end time */
 	gettimeofday(&end_time,NULL);
+#endif
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
-	broker_event_handler(NEBTYPE_EVENTHANDLER_END,NEBFLAG_NONE,NEBATTR_NONE,GLOBAL_HOST_EVENTHANDLER,(void *)hst,hst->current_state,hst->state_type,start_time,end_time,exectime,event_handler_timeout,early_timeout,result,global_host_event_handler,processed_command_line,command_output,NULL);
+	broker_event_handler(NEBTYPE_EVENTHANDLER_END,NEBFLAG_NONE,NEBATTR_NONE,GLOBAL_HOST_EVENTHANDLER,(void *)hst,hst->current_state,hst->state_type,start_time,end_time,exectime,event_handler_timeout,early_timeout,result,global_host_event_handler,processed_command,command_output,NULL);
 #endif
 
-#ifdef DEBUG0
-	printf("run_global_host_event_handler() end\n");
-#endif
+	/* free memory */
+	my_free(command_output);
+	my_free(raw_command);
+	my_free(processed_command);
 
 	return OK;
         }
@@ -534,75 +536,80 @@ int run_global_host_event_handler(host *hst){
 
 /* runs a host event handler command */
 int run_host_event_handler(host *hst){
-	char raw_command_line[MAX_COMMAND_BUFFER];
-	char processed_command_line[MAX_COMMAND_BUFFER];
-	char command_output[MAX_INPUT_BUFFER];
-	char temp_buffer[MAX_INPUT_BUFFER];
+	char *raw_command=NULL;
+	char *processed_command=NULL;
+	char *command_output=NULL;
 	int early_timeout=FALSE;
 	double exectime=0.0;
 	int result=0;
+#ifdef USE_EVENT_BROKER
 	struct timeval start_time;
 	struct timeval end_time;
+#endif
 	int macro_options=STRIP_ILLEGAL_MACRO_CHARS|ESCAPE_MACRO_CHARS;
 
-#ifdef DEBUG0
-	printf("run_host_event_handler() start\n");
-#endif
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"run_host_event_handler()\n");
+
+	if(hst==NULL)
+		return ERROR;
 
 	/* bail if there's no command */
 	if(hst->event_handler==NULL)
 		return ERROR;
 
+	log_debug_info(DEBUGL_EVENTHANDLERS,1,"Running event handler for host '%s'..\n",hst->name);
+
+#ifdef USE_EVENT_BROKER
 	/* get start time */
 	gettimeofday(&start_time,NULL);
+#endif
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
+	end_time.tv_sec=0L;
+	end_time.tv_usec=0L;
 	broker_event_handler(NEBTYPE_EVENTHANDLER_START,NEBFLAG_NONE,NEBATTR_NONE,HOST_EVENTHANDLER,(void *)hst,hst->current_state,hst->state_type,start_time,end_time,exectime,event_handler_timeout,early_timeout,result,hst->event_handler,NULL,NULL,NULL);
 #endif
 
 	/* get the raw command line */
-	get_raw_command_line(hst->event_handler,raw_command_line,sizeof(raw_command_line),macro_options);
-	strip(raw_command_line);
+	get_raw_command_line(hst->event_handler_ptr,hst->event_handler,&raw_command,macro_options);
+	if(raw_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tRaw host event handler command line: %s\n",raw_command_line);
-#endif
+	log_debug_info(DEBUGL_EVENTHANDLERS,2,"Raw host event handler command line: %s\n",raw_command);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,processed_command_line,(int)sizeof(processed_command_line),macro_options);
+	process_macros(raw_command,&processed_command,macro_options);
+	if(processed_command==NULL)
+		return ERROR;
 
-#ifdef DEBUG3
-	printf("\tProcessed host event handler command line: %s\n",processed_command_line);
-#endif
+	log_debug_info(DEBUGL_EVENTHANDLERS,2,"Processed host event handler command line: %s\n",processed_command);
 
-	if(log_event_handlers==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"HOST EVENT HANDLER: %s;%s;%s;%s;%s\n",hst->name,macro_x[MACRO_HOSTSTATE],macro_x[MACRO_HOSTSTATETYPE],macro_x[MACRO_HOSTATTEMPT],hst->event_handler);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_all_logs(temp_buffer,NSLOG_EVENT_HANDLER);
-	        }
+	if(log_event_handlers==TRUE)
+		logit(NSLOG_EVENT_HANDLER,FALSE,"HOST EVENT HANDLER: %s;%s;%s;%s;%s\n",hst->name,macro_x[MACRO_HOSTSTATE],macro_x[MACRO_HOSTSTATETYPE],macro_x[MACRO_HOSTATTEMPT],hst->event_handler);
 
 	/* run the command */
-	result=my_system(processed_command_line,event_handler_timeout,&early_timeout,&exectime,command_output,sizeof(command_output)-1);
+	result=my_system(processed_command,event_handler_timeout,&early_timeout,&exectime,&command_output,0);
 
 	/* check to see if the event handler timed out */
-	if(early_timeout==TRUE){
-		snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Host event handler command '%s' timed out after %d seconds\n",processed_command_line,event_handler_timeout);
-		temp_buffer[sizeof(temp_buffer)-1]='\x0';
-		write_to_logs_and_console(temp_buffer,NSLOG_EVENT_HANDLER | NSLOG_RUNTIME_WARNING,TRUE);
-	        }
+	if(early_timeout==TRUE)
+		logit(NSLOG_EVENT_HANDLER | NSLOG_RUNTIME_WARNING,TRUE,"Warning: Host event handler command '%s' timed out after %d seconds\n",processed_command,event_handler_timeout);
 
+#ifdef USE_EVENT_BROKER
 	/* get end time */
 	gettimeofday(&end_time,NULL);
+#endif
 
 #ifdef USE_EVENT_BROKER
 	/* send event data to broker */
-	broker_event_handler(NEBTYPE_EVENTHANDLER_END,NEBFLAG_NONE,NEBATTR_NONE,HOST_EVENTHANDLER,(void *)hst,hst->current_state,hst->state_type,start_time,end_time,exectime,event_handler_timeout,early_timeout,result,hst->event_handler,processed_command_line,command_output,NULL);
+	broker_event_handler(NEBTYPE_EVENTHANDLER_END,NEBFLAG_NONE,NEBATTR_NONE,HOST_EVENTHANDLER,(void *)hst,hst->current_state,hst->state_type,start_time,end_time,exectime,event_handler_timeout,early_timeout,result,hst->event_handler,processed_command,command_output,NULL);
 #endif
 
-#ifdef DEBUG0
-	printf("run_host_event_handler() end\n");
-#endif
+	/* free memory */
+	my_free(command_output);
+	my_free(raw_command);
+	my_free(processed_command);
 
 	return OK;
         }
@@ -618,11 +625,10 @@ int run_host_event_handler(host *hst){
 /* top level host state handler - occurs after every host check (soft/hard and active/passive) */
 int handle_host_state(host *hst){
 	int state_change=FALSE;
-	time_t current_time;
+	time_t current_time=0L;
 
-#ifdef DEBUG0
-	printf("handle_host_state() start\n");
-#endif
+
+	log_debug_info(DEBUGL_FUNCTIONS,0,"handle_host_state()\n");
 
 	/* get current time */
 	time(&current_time);
@@ -649,7 +655,7 @@ int handle_host_state(host *hst){
 	        }
 
 	/* has the host state changed? */
-	if(hst->last_state!=hst->current_state || (hst->current_state==HOST_UP && hst->state_type==SOFT_STATE))
+	if(hst->last_state!=hst->current_state || hst->last_hard_state!=hst->current_state || (hst->current_state==HOST_UP && hst->state_type==SOFT_STATE))
 		state_change=TRUE;
 
 	/* if the host state has changed... */
@@ -660,14 +666,41 @@ int handle_host_state(host *hst){
 		if(hst->state_type==HARD_STATE)
 			hst->last_hard_state_change=current_time;
 
+		/* update the event id */
+		hst->last_event_id=hst->current_event_id;
+		hst->current_event_id=next_event_id;
+		next_event_id++;
+
+		/* update the problem id when transitioning to a problem state */
+		if(hst->last_state==HOST_UP){
+			/* don't reset last problem id, or it will be zero the next time a problem is encountered */
+			/*hst->last_problem_id=hst->current_problem_id;*/
+			hst->current_problem_id=next_problem_id;
+			next_problem_id++;
+			}
+
+		/* clear the problem id when transitioning from a problem state to an UP state */
+		if(hst->current_state==HOST_UP){
+			hst->last_problem_id=hst->current_problem_id;
+			hst->current_problem_id=0L;
+			}
+
 		/* reset the acknowledgement flag if necessary */
 		if(hst->acknowledgement_type==ACKNOWLEDGEMENT_NORMAL){
+
 			hst->problem_has_been_acknowledged=FALSE;
 			hst->acknowledgement_type=ACKNOWLEDGEMENT_NONE;
+
+			/* remove any non-persistant comments associated with the ack */
+			delete_host_acknowledgement_comments(hst);
 		        }
 		else if(hst->acknowledgement_type==ACKNOWLEDGEMENT_STICKY && hst->current_state==HOST_UP){
+
 			hst->problem_has_been_acknowledged=FALSE;
 			hst->acknowledgement_type=ACKNOWLEDGEMENT_NONE;
+
+			/* remove any non-persistant comments associated with the ack */
+			delete_host_acknowledgement_comments(hst);
 		        }
 
 		/* reset the next and last notification times */
@@ -677,13 +710,6 @@ int handle_host_state(host *hst){
 		/* reset notification suppression option */
 		hst->no_more_notifications=FALSE;
 
-		/* the host just recovered, so reset the current host attempt */
-		/* 11/11/05 EG - moved below */
-		/*
-		if(hst->current_state==HOST_UP)
-			hst->current_attempt=1;
-		*/
-
 		/* write the host state change to the main log file */
 		if(hst->state_type==HARD_STATE || (hst->state_type==SOFT_STATE && log_host_retries==TRUE))
 			log_host_event(hst);
@@ -694,7 +720,7 @@ int handle_host_state(host *hst){
 
 		/* notify contacts about the recovery or problem if its a "hard" state */
 		if(hst->state_type==HARD_STATE)
-			host_notification(hst,NOTIFICATION_NORMAL,NULL,NULL);
+			host_notification(hst,NOTIFICATION_NORMAL,NULL,NULL,NOTIFICATION_OPTION_NONE);
 
 		/* handle the host state change */
 		handle_host_event(hst);
@@ -716,17 +742,13 @@ int handle_host_state(host *hst){
 
 		/* notify contacts if host is still down or unreachable */
 		if(hst->current_state!=HOST_UP && hst->state_type==HARD_STATE)
-			host_notification(hst,NOTIFICATION_NORMAL,NULL,NULL);
+			host_notification(hst,NOTIFICATION_NORMAL,NULL,NULL,NOTIFICATION_OPTION_NONE);
 
 		/* if we're in a soft state and we should log host retries, do so now... */
 		if(hst->state_type==SOFT_STATE && log_host_retries==TRUE)
 			log_host_event(hst);
 	        }
 
-#ifdef DEBUG0
-	printf("handle_host_state() end\n");
-#endif
-
 	return OK;
         }
 
