@@ -2,8 +2,8 @@
  *
  * CGIUTILS.C - Common utilities for Nagios CGIs
  * 
- * Copyright (c) 1999-2006 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 11-12-2006
+ * Copyright (c) 1999-2008 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 02-26-2008
  *
  * License:
  *
@@ -54,12 +54,21 @@ char            *normal_sound=NULL;
 char            *statusmap_background_image=NULL;
 char            *statuswrl_include=NULL;
 
+char            *illegal_output_chars=NULL;
+
+char            *notes_url_target=NULL;
+char            *action_url_target=NULL;
+
 char            *ping_syntax=NULL;
 
 char            nagios_check_command[MAX_INPUT_BUFFER]="";
 char            nagios_process_info[MAX_INPUT_BUFFER]="";
 int             nagios_process_state=STATE_OK;
 
+int             enable_splunk_integration=FALSE;
+char            *splunk_url=NULL;
+int             lock_author_names=TRUE;
+
 extern time_t   program_start;
 extern int      nagios_pid;
 extern int      daemon_mode;
@@ -89,6 +98,8 @@ int             interval_length=60;
 
 int             show_context_help=FALSE;
 
+int             use_pending_states=TRUE;
+
 int             hosts_have_been_read=FALSE;
 int             hostgroups_have_been_read=FALSE;
 int             servicegroups_have_been_read=FALSE;
@@ -101,8 +112,6 @@ int             servicedependencies_have_been_read=FALSE;
 int             serviceescalations_have_been_read=FALSE;
 int             hostdependencies_have_been_read=FALSE;
 int             hostescalations_have_been_read=FALSE;
-int             hostextinfo_have_been_read=FALSE;
-int             serviceextinfo_have_been_read=FALSE;
 
 int             host_status_has_been_read=FALSE;
 int             service_status_has_been_read=FALSE;
@@ -110,6 +119,8 @@ int             program_status_has_been_read=FALSE;
 
 int             refresh_rate=DEFAULT_REFRESH_RATE;
 
+int             escape_html_tags=FALSE;
+
 int             default_statusmap_layout_method=0;
 int             default_statuswrl_layout_method=0;
 
@@ -129,7 +140,7 @@ char            *my_strtok_buffer=NULL;
 char            *original_my_strtok_buffer=NULL;
 
 char encoded_url_string[MAX_INPUT_BUFFER];
-char encoded_html_string[MAX_INPUT_BUFFER];
+char *encoded_html_string=NULL;
 
 #ifdef HAVE_TZNAME
 #ifdef CYGWIN
@@ -263,8 +274,9 @@ char * get_cmd_file_location(void){
 /*read the CGI configuration file */
 int read_cgi_config_file(char *filename){
 	char *input=NULL;
-	char *temp_buffer;
 	mmapfile *thefile;
+	char *var=NULL;
+	char *val=NULL;
 
 
 	if((thefile=mmap_fopen(filename))==NULL)
@@ -276,63 +288,43 @@ int read_cgi_config_file(char *filename){
 		free(input);
 
 		/* read the next line */
-		if((input=mmap_fgets(thefile))==NULL)
+		if((input=mmap_fgets_multiline(thefile))==NULL)
 			break;
 
 		strip(input);
 
-		if(strstr(input,"main_config_file=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\x0");
-			if(temp_buffer!=NULL){
-				strncpy(main_config_file,temp_buffer,sizeof(main_config_file));
-				main_config_file[sizeof(main_config_file)-1]='\x0';
-				strip(main_config_file);
-			        }
+		var=strtok(input,"=");
+		val=strtok(NULL,"\n");
+
+		if(var==NULL || val==NULL)
+			continue;
+
+		if(!strcmp(var,"main_config_file")){
+			strncpy(main_config_file,val,sizeof(main_config_file));
+			main_config_file[sizeof(main_config_file)-1]='\x0';
+			strip(main_config_file);
 		        }
 
-		else if((strstr(input,"show_context_help=")==input)){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				show_context_help=TRUE;
-			else if(atoi(temp_buffer)==0)
-				show_context_help=FALSE;
-			else
-				show_context_help=TRUE;
+		else if(!strcmp(var,"show_context_help"))
+			show_context_help=(atoi(val)>0)?TRUE:FALSE;
+
+		else if(!strcmp(var,"use_pending_states"))
+			use_pending_states=(atoi(val)>0)?TRUE:FALSE;
+
+		else if(!strcmp(var,"use_authentication"))
+			use_authentication=(atoi(val)>0)?TRUE:FALSE;
+
+		else if(!strcmp(var,"nagios_check_command")){
+			strncpy(nagios_check_command,val,sizeof(nagios_check_command));
+			nagios_check_command[sizeof(nagios_check_command)-1]='\x0';
+			strip(nagios_check_command);
 		        }
 
-		else if((strstr(input,"use_authentication=")==input)){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				use_authentication=TRUE;
-			else if(atoi(temp_buffer)==0)
-				use_authentication=FALSE;
-			else
-				use_authentication=TRUE;
-		        }
+		else if(!strcmp(var,"refresh_rate"))
+			refresh_rate=atoi(val);
 
-		else if(strstr(input,"nagios_check_command=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\x0");
-			if(temp_buffer!=NULL){
-				strncpy(nagios_check_command,temp_buffer,sizeof(nagios_check_command));
-				nagios_check_command[sizeof(nagios_check_command)-1]='\x0';
-				strip(nagios_check_command);
-			        }
-		        }
-
-		else if(strstr(input,"refresh_rate=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\x0");
-			refresh_rate=atoi((temp_buffer==NULL)?"60":temp_buffer);
-		        }
-
-		else if(strstr(input,"physical_html_path=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			strncpy(physical_html_path,(temp_buffer==NULL)?"":temp_buffer,sizeof(physical_html_path));
+		else if(!strcmp(var,"physical_html_path")){
+			strncpy(physical_html_path,val,sizeof(physical_html_path));
 			physical_html_path[sizeof(physical_html_path)-1]='\x0';
 			strip(physical_html_path);
 			if(physical_html_path[strlen(physical_html_path)-1]!='/' && (strlen(physical_html_path) < sizeof(physical_html_path)-1))
@@ -345,11 +337,9 @@ int read_cgi_config_file(char *filename){
 			physical_ssi_path[sizeof(physical_ssi_path)-1]='\x0';
 		        }
 
-		else if(strstr(input,"url_html_path=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
+		else if(!strcmp(var,"url_html_path")){
 
-			strncpy(url_html_path,(temp_buffer==NULL)?"":temp_buffer,sizeof(url_html_path));
+			strncpy(url_html_path,val,sizeof(url_html_path));
 			url_html_path[sizeof(url_html_path)-1]='\x0';
 
 			strip(url_html_path);
@@ -375,90 +365,59 @@ int read_cgi_config_file(char *filename){
 			url_media_path[sizeof(url_media_path)-1]='\x0';
 		        }
 
-		else if(strstr(input,"service_critical_sound=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				continue;
-			service_critical_sound=strdup(temp_buffer);
-		        }
+		else if(!strcmp(var,"service_critical_sound"))
+			service_critical_sound=strdup(val);
 
-		else if(strstr(input,"service_warning_sound=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				continue;
-			service_warning_sound=strdup(temp_buffer);
-		        }
+		else if(!strcmp(var,"service_warning_sound"))
+			service_warning_sound=strdup(val);
 
-		else if(strstr(input,"service_unknown_sound=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				continue;
-			service_unknown_sound=strdup(temp_buffer);
-		        }
+		else if(!strcmp(var,"service_unknown_sound"))
+			service_unknown_sound=strdup(val);
 
-		else if(strstr(input,"host_down_sound=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				continue;
-			host_down_sound=strdup(temp_buffer);
-		        }
+		else if(!strcmp(var,"host_down_sound"))
+			host_down_sound=strdup(val);
 
-		else if(strstr(input,"host_unreachable_sound=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				continue;
-			host_unreachable_sound=strdup(temp_buffer);
-		        }
+		else if(!strcmp(var,"host_unreachable_sound"))
+			host_unreachable_sound=strdup(val);
 
-		else if(strstr(input,"normal_sound=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				continue;
-			normal_sound=strdup(temp_buffer);
-		        }
+		else if(!strcmp(var,"normal_sound"))
+			normal_sound=strdup(val);
 
-		else if(strstr(input,"statusmap_background_image=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				continue;
-			statusmap_background_image=strdup(temp_buffer);
-		        }
+		else if(!strcmp(var,"statusmap_background_image"))
+			statusmap_background_image=strdup(val);
 
-		else if(strstr(input,"default_statusmap_layout=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\x0");
-			default_statusmap_layout_method=atoi((temp_buffer==NULL)?"0":temp_buffer);
-		        }
+		else if(!strcmp(var,"default_statusmap_layout"))
+			default_statusmap_layout_method=atoi(val);
 
-		else if(strstr(input,"default_statuswrl_layout=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\x0");
-			default_statuswrl_layout_method=atoi((temp_buffer==NULL)?"0":temp_buffer);
-		        }
+		else if(!strcmp(var,"default_statuswrl_layout"))
+			default_statuswrl_layout_method=atoi(val);
 
-		else if(strstr(input,"statuswrl_include=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				continue;
-			statuswrl_include=strdup(temp_buffer);
-		        }
+		else if(!strcmp(var,"statuswrl_include"))
+			statuswrl_include=strdup(val);
 
-		else if(strstr(input,"ping_syntax=")==input){
-			temp_buffer=strtok(input,"=");
-			temp_buffer=strtok(NULL,"\n");
-			if(temp_buffer==NULL)
-				continue;
-			ping_syntax=strdup(temp_buffer);
-		        }
+		else if(!strcmp(var,"ping_syntax"))
+			ping_syntax=strdup(val);
 
+		else if(!strcmp(var,"action_url_target"))
+			action_url_target=strdup(val);
+
+		else if(!strcmp(var,"illegal_macro_output_chars"))
+			illegal_output_chars=strdup(val);
+
+		else if(!strcmp(var,"notes_url_target"))
+			notes_url_target=strdup(val);
+
+		else if(!strcmp(var,"enable_splunk_integration"))
+			enable_splunk_integration=(atoi(val)>0)?TRUE:FALSE;
+
+		else if(!strcmp(var,"splunk_url"))
+			splunk_url=strdup(val);
+
+		else if(!strcmp(var,"escape_html_tags"))
+			escape_html_tags=(atoi(val)>0)?TRUE:FALSE;
+
+		else if(!strcmp(var,"lock_author_names"))
+			lock_author_names=(atoi(val)>0)?TRUE:FALSE;
  	        }
 
 	/* free memory and close the file */
@@ -489,7 +448,7 @@ int read_main_config_file(char *filename){
 		free(input);
 
 		/* read the next line */
-		if((input=mmap_fgets(thefile))==NULL)
+		if((input=mmap_fgets_multiline(thefile))==NULL)
 			break;
 
 		strip(input);
@@ -599,11 +558,7 @@ int read_all_object_configuration_data(char *config_file,int options){
 		options-=READ_HOSTDEPENDENCIES;
 	if(hostescalations_have_been_read==TRUE && (options & READ_HOSTESCALATIONS))
 		options-=READ_HOSTESCALATIONS;
-	if(hostextinfo_have_been_read==TRUE && (options & READ_HOSTEXTINFO))
-		options-=READ_HOSTEXTINFO;
-	if(serviceextinfo_have_been_read==TRUE && (options & READ_SERVICEEXTINFO))
-		options-=READ_SERVICEEXTINFO;
-	if(serviceextinfo_have_been_read==TRUE && (options & READ_SERVICEGROUPS))
+	if(servicegroups_have_been_read==TRUE && (options & READ_SERVICEGROUPS))
 		options-=READ_SERVICEGROUPS;
 
 	/* bail out if we've already read what we need */
@@ -611,7 +566,7 @@ int read_all_object_configuration_data(char *config_file,int options){
 		return OK;
 
 	/* read in all external config data of the desired type(s) */
-	result=read_object_config_data(config_file,options,FALSE);
+	result=read_object_config_data(config_file,options,FALSE,FALSE);
 
 	/* mark what items we've read in... */
 	if(options & READ_HOSTS)
@@ -636,10 +591,6 @@ int read_all_object_configuration_data(char *config_file,int options){
 		hostdependencies_have_been_read=TRUE;
 	if(options & READ_HOSTESCALATIONS)
 		hostescalations_have_been_read=TRUE;
-	if(options & READ_HOSTEXTINFO)
-		hostextinfo_have_been_read=TRUE;
-	if(options & READ_SERVICEEXTINFO)
-		serviceextinfo_have_been_read=TRUE;
 	if(options & READ_SERVICEGROUPS)
 		servicegroups_have_been_read=TRUE;
 
@@ -684,27 +635,12 @@ int read_all_status_data(char *config_file,int options){
  **********************************************************/
 
 
-/* single hash function */
-int hashfunc1(const char *name1,int hashslots){
-	unsigned int i,result;
-
-	result=0;
-
-	if(name1)
-		for(i=0;i<strlen(name1);i++)
-			result+=name1[i];
-
-	result=result%hashslots;
-
-	return result;
-        }
-
-
 /* dual hash function */
-int hashfunc2(const char *name1,const char *name2,int hashslots){
+int hashfunc(const char *name1,const char *name2,int hashslots){
 	unsigned int i,result;
 
 	result=0;
+
 	if(name1)
 		for(i=0;i<strlen(name1);i++)
 			result+=name1[i];
@@ -719,26 +655,36 @@ int hashfunc2(const char *name1,const char *name2,int hashslots){
         }
 
 
-/* single hash data comparison */
-int compare_hashdata1(const char *val1, const char *val2){
-	
-	return strcmp(val1,val2);
-        }
-
-
 /* dual hash data comparison */
-int compare_hashdata2(const char *val1a, const char *val1b, const char *val2a, const char *val2b){
-	int result;
+int compare_hashdata(const char *val1a, const char *val1b, const char *val2a, const char *val2b){
+	int result=0;
 
-	result=strcmp(val1a,val2a);
-	if(result>0)
-		return 1;
-	else if(result<0)
-		return -1;
+	/* NOTE: If hash calculation changes, update the compare_strings() function! */
+
+	/* check first name */
+	if(val1a==NULL && val2a==NULL)
+		result=0;
+	else if(val1a==NULL)
+		result=1;
+	else if(val2a==NULL)
+		result=-1;
 	else
-		return strcmp(val1b,val2b);
-        }
+		result=strcmp(val1a,val2a);
 
+	/* check second name if necessary */
+	if(result==0){
+		if(val1b==NULL && val2b==NULL)
+			result=0;
+		else if(val1b==NULL)
+			result=1;
+		else if(val2b==NULL)
+			result=-1;
+		else
+			result=strcmp(val1b,val2b);
+	        }
+
+	return result;
+        }
 
 
 
@@ -891,6 +837,36 @@ void strip(char *buffer){
 	}
 
 
+/* unescapes newlines in a string */
+char *unescape_newlines(char *rawbuf){
+	register int x,y;
+
+	for(x=0,y=0;rawbuf[x]!=(char)'\x0';x++){
+
+		if(rawbuf[x]=='\\'){
+
+			/* unescape newlines */
+			if(rawbuf[x+1]=='n'){
+				rawbuf[y++]='\n';
+				x++;
+				}
+
+			/* unescape backslashes and other stuff */
+			else{
+				rawbuf[y++]=rawbuf[x+1];
+				x++;
+				}
+
+			}
+		else
+			rawbuf[y++]=rawbuf[x];
+		}
+	rawbuf[y]='\x0';
+
+	return rawbuf;
+	}
+
+
 /* strips HTML and bad stuff from plugin output */
 void sanitize_plugin_output(char *buffer){
 	int x=0;
@@ -958,12 +934,13 @@ void sanitize_plugin_output(char *buffer){
 /* get date/time string */
 void get_time_string(time_t *raw_time,char *buffer,int buffer_length,int type){
 	time_t t;
-	struct tm *tm_ptr;
-	int day;
-	int hour;
-	int minute;
-	int second;
-	int year;
+	struct tm *tm_ptr=NULL;
+	int hour=0;
+	int minute=0;
+	int second=0;
+	int month=0;
+	int day=0;
+	int year=0;
 	char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
 	char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
 	char *tzone="";
@@ -981,6 +958,7 @@ void get_time_string(time_t *raw_time,char *buffer,int buffer_length,int type){
 	hour=tm_ptr->tm_hour;
 	minute=tm_ptr->tm_min;
 	second=tm_ptr->tm_sec;
+	month=tm_ptr->tm_mon+1;
 	day=tm_ptr->tm_mday;
 	year=tm_ptr->tm_year+1900;
 
@@ -1004,18 +982,103 @@ void get_time_string(time_t *raw_time,char *buffer,int buffer_length,int type){
 			snprintf(buffer,buffer_length,"%02d-%02d-%04d %02d:%02d:%02d",tm_ptr->tm_mon+1,tm_ptr->tm_mday,tm_ptr->tm_year+1900,tm_ptr->tm_hour,tm_ptr->tm_min,tm_ptr->tm_sec);
 	        }
 
+	/* short date */
+	else if(type==SHORT_DATE){
+		if(date_format==DATE_FORMAT_EURO)
+			snprintf(buffer,buffer_length,"%02d-%02d-%04d",day,month,year);
+		else if(date_format==DATE_FORMAT_ISO8601 || date_format==DATE_FORMAT_STRICT_ISO8601)
+			snprintf(buffer,buffer_length,"%04d-%02d-%02d",year,month,day);
+		else
+			snprintf(buffer,buffer_length,"%02d-%02d-%04d",month,day,year);
+	        }
+
 	/* expiration date/time for HTTP headers */
 	else if(type==HTTP_DATE_TIME)
 		snprintf(buffer,buffer_length,"%s, %02d %s %d %02d:%02d:%02d GMT",weekdays[tm_ptr->tm_wday],day,months[tm_ptr->tm_mon],year,hour,minute,second);
 
+	/* short time */
+	else
+		snprintf(buffer,buffer_length,"%02d:%02d:%02d",hour,minute,second);
+
 	buffer[buffer_length-1]='\x0';
 
 	return;
         }
 
 
+
+/* given a date/time in time_t format, produce a corresponding date/time string, including timezone */
+void get_datetime_string(time_t *raw_time,char *buffer,int buffer_length, int type){
+	time_t t;
+	struct tm *tm_ptr=NULL;
+	int hour=0;
+	int minute=0;
+	int second=0;
+	int month=0;
+	int day=0;
+	int year=0;
+	char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
+	char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sept","Oct","Nov","Dec"};
+
+#ifdef HAVE_TM_ZONE
+# define tzone tm_ptr->tm_zone
+#else
+# define tzone (tm_ptr->tm_isdst)?tzname[1]:tzname[0]
+#endif
+
+	if(raw_time==NULL)
+		time(&t);
+	else 
+		t=*raw_time;
+
+	tm_ptr=localtime(&t);
+
+	hour=tm_ptr->tm_hour;
+	minute=tm_ptr->tm_min;
+	second=tm_ptr->tm_sec;
+	month=tm_ptr->tm_mon+1;
+	day=tm_ptr->tm_mday;
+	year=tm_ptr->tm_year+1900;
+
+	/* ctime() style date/time */
+	if(type==LONG_DATE_TIME)
+		snprintf(buffer,buffer_length,"%s %s %d %02d:%02d:%02d %s %d",weekdays[tm_ptr->tm_wday],months[tm_ptr->tm_mon],day,hour,minute,second,tzone,year);
+
+	/* short date/time */
+	else if(type==SHORT_DATE_TIME){
+		if(date_format==DATE_FORMAT_EURO)
+			snprintf(buffer,buffer_length,"%02d-%02d-%04d %02d:%02d:%02d",day,month,year,hour,minute,second);
+		else if(date_format==DATE_FORMAT_ISO8601 || date_format==DATE_FORMAT_STRICT_ISO8601)
+			snprintf(buffer,buffer_length,"%04d-%02d-%02d%c%02d:%02d:%02d",year,month,day,(date_format==DATE_FORMAT_STRICT_ISO8601)?'T':' ',hour,minute,second);
+		else
+			snprintf(buffer,buffer_length,"%02d-%02d-%04d %02d:%02d:%02d",month,day,year,hour,minute,second);
+	        }
+
+	/* short date */
+	else if(type==SHORT_DATE){
+		if(date_format==DATE_FORMAT_EURO)
+			snprintf(buffer,buffer_length,"%02d-%02d-%04d",day,month,year);
+		else if(date_format==DATE_FORMAT_ISO8601 || date_format==DATE_FORMAT_STRICT_ISO8601)
+			snprintf(buffer,buffer_length,"%04d-%02d-%02d",year,month,day);
+		else
+			snprintf(buffer,buffer_length,"%02d-%02d-%04d",month,day,year);
+	        }
+
+	/* short time */
+	else
+		snprintf(buffer,buffer_length,"%02d:%02d:%02d",hour,minute,second);
+
+	buffer[buffer_length-1]='\x0';
+
+	/* don't mess up other functions that might want to call a variable 'tzone' */
+#undef tzone
+
+	return;
+        }
+
+
 /* get time string for an interval of time */
-void get_interval_time_string(int time_units,char *buffer,int buffer_length){
+void get_interval_time_string(double time_units,char *buffer,int buffer_length){
 	unsigned long total_seconds;
 	int hours=0;
 	int minutes=0;
@@ -1112,6 +1175,23 @@ char *my_strsep (char **stringp, const char *delim){
 	}
 
 
+#ifdef REMOVED_10182007
+/* my wrapper for free() */
+int my_free(void **ptr){
+
+	if(ptr==NULL)
+		return ERROR;
+
+	/* I hate calling free() and then resetting the pointer to NULL, so lets do it together */
+	if(*ptr){
+		free(*ptr);
+		*ptr=NULL;
+	        }
+
+	return OK;
+        }
+#endif
+
 
 /* open a file read-only via mmap() */
 mmapfile *mmap_fopen(char *filename){
@@ -1218,19 +1298,21 @@ char *mmap_fgets(mmapfile *temp_mmapfile){
         }
 
 
+
 /* gets one line of input from an mmap()'ed file (may be contained on more than one line in the source file) */
 char *mmap_fgets_multiline(mmapfile *temp_mmapfile){
 	char *buf=NULL;
 	char *tempbuf=NULL;
-	int len;
-	int len2;
+	int len=0;
+	int len2=0;
+	int end=0;
 
 	if(temp_mmapfile==NULL)
 		return NULL;
 
 	while(1){
 
-		free(tempbuf);
+		my_free(tempbuf);
 
 		if((tempbuf=mmap_fgets(temp_mmapfile))==NULL)
 			break;
@@ -1252,12 +1334,35 @@ char *mmap_fgets_multiline(mmapfile *temp_mmapfile){
 			buf[len]='\x0';
 		        }
 
-		/* we shouldn't continue to the next line... */
-		if(!(len>0 && buf[len-1]=='\\' && (len==1 || buf[len-2]!='\\')))
+		if(len==0)
+			break;
+
+		/* handle Windows/DOS CR/LF */
+		if(len>=2 && buf[len-2]=='\r')
+			end=len-3;
+		/* normal Unix LF */
+		else if(len>=1 && buf[len-1]=='\n')
+			end=len-2;
+		else
+			end=len-1;
+
+		/* two backslashes found, so unescape first backslash first and break */
+		if(end>=1 && buf[end-1]=='\\' && buf[end]=='\\'){
+			buf[end]='\n';
+			buf[end+1]='\x0';
+			break;
+			}
+
+		/* one backslash found, so we should continue reading the next line */
+		else if(buf[end]=='\\')
+			buf[end]='\x0';
+
+		/* else no continuation marker was found, so break */
+		else
 			break;
 	        }
 
-	free(tempbuf);
+	my_free(tempbuf);
 
 	return buf;
         }
@@ -1297,6 +1402,7 @@ char * url_encode(char *input){
 	int len,output_len;
 	int x,y;
 	char temp_expansion[4];
+	int expansion_len=0;
 
 	len=(int)strlen(input);
 	output_len=(int)sizeof(encoded_url_string);
@@ -1318,18 +1424,21 @@ char * url_encode(char *input){
 		        }
 
 		/* spaces are pluses */
+		/*
 		else if((char)input[x]<=(char)' '){
 			encoded_url_string[y]='+';
 			y++;
 		        }
+		*/
 
 		/* anything else gets represented by its hex value */
 		else{
 			encoded_url_string[y]='\x0';
-			if((int)strlen(encoded_url_string)<(output_len-3)){
-				sprintf(temp_expansion,"%%%02X",(unsigned int)input[x]);
+			sprintf(temp_expansion,"%%%02X",(unsigned int)(input[x] & 0xFF));
+			expansion_len=strlen(temp_expansion);
+			if((int)strlen(encoded_url_string)<(output_len-expansion_len)){
 				strcat(encoded_url_string,temp_expansion);
-				y+=3;
+				y+=expansion_len;
 			        }
 		        }
 	        }
@@ -1342,17 +1451,20 @@ char * url_encode(char *input){
 
 
 /* escapes a string used in HTML */
-char * html_encode(char *input){
+char * html_encode(char *input, int escape_newlines){
 	int len,output_len;
 	int x,y;
-	char temp_expansion[7];
+	char temp_expansion[10];
 
+	/* we need up to six times the space to do the conversion */
 	len=(int)strlen(input);
-	output_len=(int)sizeof(encoded_html_string);
+	output_len=len*6;
+	if((encoded_html_string=(char *)malloc(output_len+1))==NULL)
+		return "";
 
-	encoded_html_string[0]='\x0';
+	strcpy(encoded_html_string,"");
 
-	for(x=0,y=0;x<=len && y<output_len-1;x++){
+	for(x=0,y=0;x<=len;x++){
 
 		/* end of string */
 		if((char)input[x]==(char)'\x0'){
@@ -1360,40 +1472,94 @@ char * html_encode(char *input){
 			break;
 		        }
 
-		/* alpha-numeric characters don't get encoded */
-		else if(((char)input[x]>='0' && (char)input[x]<='9') || ((char)input[x]>='A' && (char)input[x]<='Z') || ((char)input[x]>=(char)'a' && (char)input[x]<=(char)'z')){
-			encoded_html_string[y]=input[x];
-			y++;
+		/* alpha-numeric characters and spaces don't get encoded */
+		else if(((char)input[x]==(char)' ') || ((char)input[x]>='0' && (char)input[x]<='9') || ((char)input[x]>='A' && (char)input[x]<='Z') || ((char)input[x]>=(char)'a' && (char)input[x]<=(char)'z'))
+			encoded_html_string[y++]=input[x];
+
+		/* newlines turn to <BR> tags */
+		else if(escape_newlines==TRUE && (char)input[x]==(char)'\n'){
+			strcpy(&encoded_html_string[y],"<BR>");
+			y+=4;
+			}
+		else if(escape_newlines==TRUE && (char)input[x]==(char)'\\' && (char)input[x+1]==(char)'n'){
+			strcpy(&encoded_html_string[y],"<BR>");
+			y+=4;
+			x++;
+			}
+
+		/* TODO - strip all but allowed HTML tags out... */
+
+		else if((char)input[x]==(char)'<'){
+
+			if(escape_html_tags==FALSE)
+				encoded_html_string[y++]=input[x];
+			else{
+				encoded_html_string[y]='\x0';
+				if((int)strlen(encoded_html_string)<(output_len-4)){
+					strcat(encoded_html_string,"&lt;");
+					y+=4;
+					}
+				}
 		        }
 
-		/* spaces are encoded as non-breaking spaces */
-		else if((char)input[x]<=(char)' '){
+		else if((char)input[x]==(char)'>'){
 
-			encoded_html_string[y]='\x0';
-			if((int)strlen(encoded_html_string)<(output_len-6)){
-				strcat(encoded_html_string,"&nbsp;");
-				y+=6;
-			        }
+			if(escape_html_tags==FALSE)
+				encoded_html_string[y++]=input[x];
+			else{
+				encoded_html_string[y]='\x0';
+				if((int)strlen(encoded_html_string)<(output_len-4)){
+					strcat(encoded_html_string,"&gt;");
+					y+=4;
+					}
+				}
 		        }
 
-		/* for simplicity, everything else gets represented by its numeric value */
+		/* for simplicity, all other chars represented by their numeric value */
 		else{
-			encoded_html_string[y]='\x0';
-			sprintf(temp_expansion,"&#%d;",(unsigned int)input[x]);
-			if((int)strlen(encoded_html_string)<(output_len-strlen(temp_expansion))){
-				strcat(encoded_html_string,temp_expansion);
-				y+=strlen(temp_expansion);
-			        }
+			if(escape_html_tags==FALSE)
+				encoded_html_string[y++]=input[x];
+			else{
+				encoded_html_string[y]='\x0';
+				sprintf(temp_expansion,"&#%d;",(unsigned char)input[x]);
+				if((int)strlen(encoded_html_string)<(output_len-strlen(temp_expansion))){
+					strcat(encoded_html_string,temp_expansion);
+					y+=strlen(temp_expansion);
+					}
+				}
 		        }
 	        }
 
-	encoded_html_string[sizeof(encoded_html_string)-1]='\x0';
+	encoded_html_string[y++]='\x0';
 
-	return &encoded_html_string[0];
+	return encoded_html_string;
         }
 
 
 
+/* strip > and < from string */
+void strip_html_brackets(char *buffer){
+	register int x;
+	register int y;
+	register int z;
+
+	if(buffer==NULL || buffer[0]=='\x0')
+		return;
+
+	/* remove all occurances in string */
+	z=(int)strlen(buffer);
+	for(x=0,y=0;x<z;x++){
+		if(buffer[x]=='<' || buffer[x]=='>')
+			continue;
+		buffer[y++]=buffer[x];
+	        }
+	buffer[y++]='\x0';
+
+	return;
+	}
+
+
+
 /* determines the log file we should use (from current time) */
 void get_log_archive_to_use(int archive,char *buffer,int buffer_length){
 	struct tm *t;
@@ -1574,7 +1740,7 @@ void display_info_table(char *title,int refresh, authdata *current_authdata){
 	if(refresh==TRUE)
 		printf("Updated every %d seconds<br>\n",refresh_rate);
 
-	printf("Nagios&reg; - <A HREF='http://www.nagios.org' TARGET='_new' CLASS='homepageURL'>www.nagios.org</A><BR>\n");
+	printf("Nagios&reg; %s - <A HREF='http://www.nagios.org' TARGET='_new' CLASS='homepageURL'>www.nagios.org</A><BR>\n",PROGRAM_VERSION);
 
 	if(current_authdata!=NULL)
 		printf("Logged in as <i>%s</i><BR>\n",(!strcmp(current_authdata->username,""))?"?":current_authdata->username);
@@ -1673,19 +1839,19 @@ void display_nav_table(char *url,int archive){
 
 
 
-/* prints the additional notes or action url for a host (with macros substituted) */
-void print_extra_host_url(char *host_name, char *url){
+/* prints the additional notes or action url for a hostgroup (with macros substituted) */
+void print_extra_hostgroup_url(char *group_name, char *url){
 	char input_buffer[MAX_INPUT_BUFFER]="";
 	char output_buffer[MAX_INPUT_BUFFER]="";
 	char *temp_buffer;
 	int in_macro=FALSE;
-	host *temp_host;
+	hostgroup *temp_hostgroup=NULL;
 
-	if(host_name==NULL || url==NULL)
+	if(group_name==NULL || url==NULL)
 		return;
 
-	temp_host=find_host(host_name);
-	if(temp_host==NULL){
+	temp_hostgroup=find_hostgroup(group_name);
+	if(temp_hostgroup==NULL){
 		printf("%s",url);
 		return;
 	        }
@@ -1706,11 +1872,8 @@ void print_extra_host_url(char *host_name, char *url){
 
 			if(strlen(output_buffer)+strlen(temp_buffer) < sizeof(output_buffer)-1){
 
-				if(!strcmp(temp_buffer,"HOSTNAME"))
-					strncat(output_buffer,url_encode(temp_host->name),sizeof(output_buffer)-strlen(output_buffer)-1);
-
-				else if(!strcmp(temp_buffer,"HOSTADDRESS"))
-					strncat(output_buffer,(temp_host->address==NULL)?"":url_encode(temp_host->address),sizeof(output_buffer)-strlen(output_buffer)-1);
+				if(!strcmp(temp_buffer,"HOSTGROUPNAME"))
+					strncat(output_buffer,url_encode(temp_hostgroup->group_name),sizeof(output_buffer)-strlen(output_buffer)-1);
 			        }
 
 			in_macro=FALSE;
@@ -1724,28 +1887,25 @@ void print_extra_host_url(char *host_name, char *url){
 
 
 
-/* prints the additional notes or action url for a service (with macros substituted) */
-void print_extra_service_url(char *host_name, char *svc_description, char *url){
+/* prints the additional notes or action url for a servicegroup (with macros substituted) */
+void print_extra_servicegroup_url(char *group_name, char *url){
 	char input_buffer[MAX_INPUT_BUFFER]="";
 	char output_buffer[MAX_INPUT_BUFFER]="";
 	char *temp_buffer;
 	int in_macro=FALSE;
-	service *temp_service;
-	host *temp_host;
+	servicegroup *temp_servicegroup=NULL;
 
-	if(host_name==NULL || svc_description==NULL || url==NULL)
+	if(group_name==NULL || url==NULL)
 		return;
 
-	temp_service=find_service(host_name,svc_description);
-	if(temp_service==NULL){
+	temp_servicegroup=find_servicegroup(group_name);
+	if(temp_servicegroup==NULL){
 		printf("%s",url);
 		return;
 	        }
 
-	temp_host=find_host(host_name);
-
 	strncpy(input_buffer,url,sizeof(input_buffer)-1);
-	input_buffer[sizeof(input_buffer)-1]='\x0';
+	output_buffer[sizeof(input_buffer)-1]='\x0';
 
 	for(temp_buffer=my_strtok(input_buffer,"$");temp_buffer!=NULL;temp_buffer=my_strtok(NULL,"$")){
 
@@ -1760,14 +1920,8 @@ void print_extra_service_url(char *host_name, char *svc_description, char *url){
 
 			if(strlen(output_buffer)+strlen(temp_buffer) < sizeof(output_buffer)-1){
 
-				if(!strcmp(temp_buffer,"HOSTNAME"))
-					strncat(output_buffer,url_encode(temp_service->host_name),sizeof(output_buffer)-strlen(output_buffer)-1);
-
-				else if(!strcmp(temp_buffer,"HOSTADDRESS") && temp_host!=NULL)
-					strncat(output_buffer,(temp_host->address==NULL)?"":url_encode(temp_host->address),sizeof(output_buffer)-strlen(output_buffer)-1);
-
-				else if(!strcmp(temp_buffer,"SERVICEDESC"))
-					strncat(output_buffer,url_encode(temp_service->description),sizeof(output_buffer)-strlen(output_buffer)-1);
+				if(!strcmp(temp_buffer,"SERVICEGROUPNAME"))
+					strncat(output_buffer,url_encode(temp_servicegroup->group_name),sizeof(output_buffer)-strlen(output_buffer)-1);
 			        }
 
 			in_macro=FALSE;
@@ -1781,7 +1935,6 @@ void print_extra_service_url(char *host_name, char *svc_description, char *url){
 
 
 
-
 /* include user-defined SSI footers or headers */
 void include_ssi_files(char *cgi_name, int type){
 	char common_ssi_file[MAX_INPUT_BUFFER];
@@ -1804,14 +1957,14 @@ void include_ssi_files(char *cgi_name, int type){
 		cgi_ssi_file[x]=tolower(cgi_ssi_file[x]);
 
 	if(type==SSI_HEADER){
-		printf("\n<!-- Produced by Nagios (http://www.nagios.org).  Copyright (c) 1999-2003 Ethan Galstad. -->\n");
+		printf("\n<!-- Produced by Nagios (http://www.nagios.org).  Copyright (c) 1999-2007 Ethan Galstad. -->\n");
 		include_ssi_file(common_ssi_file);
 		include_ssi_file(cgi_ssi_file);
 	        }
 	else{
 		include_ssi_file(cgi_ssi_file);
 		include_ssi_file(common_ssi_file);
-		printf("\n<!-- Produced by Nagios (http://www.nagios.org).  Copyright (c) 1999-2003 Ethan Galstad. -->\n");
+		printf("\n<!-- Produced by Nagios (http://www.nagios.org).  Copyright (c) 1999-2007 Ethan Galstad. -->\n");
 	        }
 
 	return;
@@ -1954,7 +2107,6 @@ void object_data_error(void){
 
 	printf("<LI>Verify configuration options using the <b>-v</b> command-line option to check for errors.\n");
 	printf("<LI>Check the Nagios log file for messages relating to startup or status data errors.\n");
-	printf("<LI>Make sure you've compiled the main program and the CGIs to use the same object data storage options (i.e. default text file or template-based file).\n");
 
 	printf("</OL>\n");
 	printf("</P>\n");
@@ -1987,7 +2139,6 @@ void status_data_error(void){
 
 	printf("<LI>Check the Nagios log file for messages relating to startup or status data errors.\n");
 	printf("<LI>Always verify configuration options using the <b>-v</b> command-line option before starting or restarting Nagios!\n");
-	printf("<LI>Make sure you've compiled the main program and the CGIs to use the same status data storage options (i.e. text file or database).  If the main program is storing status data in a text file and the CGIs are trying to read status data from a database, you'll have problems.\n");
 
 	printf("</OL>\n");
 	printf("</P>\n");
@@ -2017,3 +2168,82 @@ void display_context_help(char *chid){
 
 	return;
         }
+
+
+
+void display_splunk_host_url(host *hst){
+
+	if(enable_splunk_integration==FALSE)
+		return;
+	if(hst==NULL)
+		return;
+
+	printf("<a href='%s?q=%s' target='_blank'><img src='%s%s' alt='Splunk It' title='Splunk It' border='0'></a>\n",splunk_url,url_encode(hst->name),url_images_path,SPLUNK_SMALL_WHITE_ICON);
+
+	return;
+	}
+
+
+
+void display_splunk_service_url(service *svc){
+
+	if(enable_splunk_integration==FALSE)
+		return;
+	if(svc==NULL)
+		return;
+
+	printf("<a href='%s?q=%s%%20",splunk_url,url_encode(svc->host_name));
+	printf("%s' target='_blank'><img src='%s%s' alt='Splunk It' title='Splunk It' border='0'></a>\n",url_encode(svc->description),url_images_path,SPLUNK_SMALL_WHITE_ICON);
+
+	return;
+	}
+
+
+
+void display_splunk_generic_url(char *buf, int icon){
+	char *newbuf=NULL;
+
+	if(enable_splunk_integration==FALSE)
+		return;
+	if(buf==NULL)
+		return;
+
+	if((newbuf=(char *)strdup(buf))==NULL)
+		return;
+
+	strip_splunk_query_terms(newbuf);
+
+	printf("<a href='%s?q=%s' target='_blank'>",splunk_url,url_encode(newbuf));
+	if(icon>0)
+		printf("<img src='%s%s' alt='Splunk It' title='Splunk It' border='0'>",url_images_path,(icon==1)?SPLUNK_SMALL_WHITE_ICON:SPLUNK_SMALL_BLACK_ICON);
+	printf("</a>\n");
+
+	free(newbuf);
+
+	return;
+	}
+
+
+/* strip quotes and from string */
+void strip_splunk_query_terms(char *buffer){
+	register int x;
+	register int y;
+	register int z;
+
+	if(buffer==NULL || buffer[0]=='\x0')
+		return;
+
+	/* remove all occurances in string */
+	z=(int)strlen(buffer);
+	for(x=0,y=0;x<z;x++){
+		if(buffer[x]=='\'' || buffer[x]=='\"' || buffer[x]==';' || buffer[x]==':' || buffer[x]==',' || buffer[x]=='-' || buffer[x]=='=')
+			buffer[y++]=' ';
+		else
+			buffer[y++]=buffer[x];
+	        }
+	buffer[y++]='\x0';
+
+	return;
+	}
+
+
