@@ -2,8 +2,8 @@
  *
  * HISTOGRAM.C -  Nagios Alert Histogram CGI
  *
- * Copyright (c) 2001-2007 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 10-21-2007
+ * Copyright (c) 2001-2008 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 01-08-2008
  *
  * License:
  * 
@@ -125,6 +125,7 @@
 
 
 extern char main_config_file[MAX_FILENAME_LENGTH];
+extern char url_html_path[MAX_FILENAME_LENGTH];
 extern char url_images_path[MAX_FILENAME_LENGTH];
 extern char url_stylesheets_path[MAX_FILENAME_LENGTH];
 extern char physical_images_path[MAX_FILENAME_LENGTH];
@@ -1000,6 +1001,7 @@ void document_header(int use_stylesheet){
 
 		printf("<html>\n");
 		printf("<head>\n");
+		printf("<link rel=\"shortcut icon\" href=\"%sfavicon.ico\" type=\"image/ico\">\n",url_images_path);
 		printf("<title>\n");
 		printf("Nagios Histogram\n");
 		printf("</title>\n");
@@ -1081,11 +1083,8 @@ int process_cgivars(void){
 				break;
 			        }
 
-			host_name=(char *)malloc(strlen(variables[x])+1);
-			if(host_name==NULL)
+			if((host_name=(char *)strdup(variables[x]))==NULL)
 				host_name="";
-			else
-				strcpy(host_name,variables[x]);
 			strip_html_brackets(host_name);
 
 			display_type=DISPLAY_HOST_HISTOGRAM;
@@ -1099,11 +1098,8 @@ int process_cgivars(void){
 				break;
 			        }
 
-			svc_description=(char *)malloc(strlen(variables[x])+1);
-			if(svc_description==NULL)
+			if((svc_description=(char *)strdup(variables[x]))==NULL)
 				svc_description="";
-			else
-				strcpy(svc_description,variables[x]);
 			strip_html_brackets(svc_description);
 
 			display_type=DISPLAY_SERVICE_HISTOGRAM;
