@@ -2,8 +2,8 @@
  *
  * HISTORY.C - Nagios History CGI
  *
- * Copyright (c) 1999-2007 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 04-10-2007
+ * Copyright (c) 1999-2008 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 01-08-2008
  *
  * This CGI program will display the history for the specified host.
  * If no host is specified, the history for all hosts will be displayed.
@@ -53,11 +53,14 @@ void document_footer(void);
 int process_cgivars(void);
 
 extern char main_config_file[MAX_FILENAME_LENGTH];
+extern char url_html_path[MAX_FILENAME_LENGTH];
 extern char url_images_path[MAX_FILENAME_LENGTH];
 extern char url_stylesheets_path[MAX_FILENAME_LENGTH];
 
 extern int log_rotation_method;
 
+extern int enable_splunk_integration;
+
 authdata current_authdata;
 
 char log_file_to_use[MAX_FILENAME_LENGTH];
@@ -319,6 +322,7 @@ void document_header(int use_stylesheet){
 
 	printf("<html>\n");
 	printf("<head>\n");
+	printf("<link rel=\"shortcut icon\" href=\"%sfavicon.ico\" type=\"image/ico\">\n",url_images_path);
 	printf("<title>\n");
 	printf("Nagios History\n");
 	printf("</title>\n");
@@ -374,11 +378,9 @@ int process_cgivars(void){
 				break;
 			        }
 
-			host_name=(char *)malloc(strlen(variables[x])+1);
-			if(host_name==NULL)
+			if((host_name=(char *)strdup(variables[x]))==NULL)
 				host_name="";
-			else
-				strcpy(host_name,variables[x]);
+			strip_html_brackets(host_name);
 
 			display_type=DISPLAY_HOSTS;
 
@@ -396,11 +398,9 @@ int process_cgivars(void){
 				break;
 			        }
 
-			svc_description=(char *)malloc(strlen(variables[x])+1);
-			if(svc_description==NULL)
+			if((svc_description=(char *)strdup(variables[x]))==NULL)
 				svc_description="";
-			else
-				strcpy(svc_description,variables[x]);
+			strip_html_brackets(svc_description);
 
 			display_type=DISPLAY_SERVICES;
 		        }
@@ -485,7 +485,7 @@ int process_cgivars(void){
 
 
 void get_history(void){
-	mmapfile *thefile;
+	mmapfile *thefile=NULL;
 	char image[MAX_INPUT_BUFFER];
 	char image_alt[MAX_INPUT_BUFFER];
 	char *input=NULL;
@@ -497,18 +497,18 @@ void get_history(void){
 	int display_line=FALSE;
 	time_t t;
 	char date_time[MAX_DATETIME_LENGTH];
-	char *temp_buffer;
+	char *temp_buffer=NULL;
 	int history_type=SERVICE_HISTORY;
 	int history_detail_type=HISTORY_SERVICE_CRITICAL;
-	char entry_host_name[MAX_HOSTNAME_LENGTH];
-	char entry_service_desc[MAX_SERVICEDESC_LENGTH];
-	host *temp_host;
-	service *temp_service;
-	int result;
+	char *entry_host_name=NULL;
+	char *entry_service_desc=NULL;
+	host *temp_host=NULL;
+	service *temp_service=NULL;
+	int result=0;
 
 	char last_message_date[MAX_INPUT_BUFFER]="";
 	char current_message_date[MAX_INPUT_BUFFER]="";
-	struct tm *time_ptr;
+	struct tm *time_ptr=NULL;
 
 
 	if(use_lifo==TRUE){
@@ -565,11 +565,15 @@ void get_history(void){
 			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
-			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
-			entry_host_name[sizeof(entry_host_name)-1]='\x0';
+			if(temp_buffer)
+				entry_host_name=strdup(temp_buffer+1);
+			else
+				entry_host_name=NULL;
 			temp_buffer=my_strtok(NULL,";");
-			strncpy(entry_service_desc,(temp_buffer==NULL)?"":temp_buffer,sizeof(entry_service_desc));
-			entry_service_desc[sizeof(entry_service_desc)-1]='\x0';
+			if(temp_buffer)
+				entry_service_desc=strdup(temp_buffer);
+			else
+				entry_service_desc=NULL;
 
 			if(strstr(input,";CRITICAL;")){
 				strncpy(image,CRITICAL_ICON,sizeof(image));
@@ -605,11 +609,15 @@ void get_history(void){
 			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
-			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
-			entry_host_name[sizeof(entry_host_name)-1]='\x0';
+			if(temp_buffer)
+				entry_host_name=strdup(temp_buffer+1);
+			else
+				entry_host_name=NULL;
 			temp_buffer=my_strtok(NULL,";");
-			strncpy(entry_service_desc,(temp_buffer==NULL)?"":temp_buffer,sizeof(entry_service_desc));
-			entry_service_desc[sizeof(entry_service_desc)-1]='\x0';
+			if(temp_buffer)
+				entry_service_desc=strdup(temp_buffer);
+			else
+				entry_service_desc=NULL;
 
 			strncpy(image,FLAPPING_ICON,sizeof(image));
 
@@ -633,11 +641,15 @@ void get_history(void){
 			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
-			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
-			entry_host_name[sizeof(entry_host_name)-1]='\x0';
+			if(temp_buffer)
+				entry_host_name=strdup(temp_buffer+1);
+			else
+				entry_host_name=NULL;
 			temp_buffer=my_strtok(NULL,";");
-			strncpy(entry_service_desc,(temp_buffer==NULL)?"":temp_buffer,sizeof(entry_service_desc));
-			entry_service_desc[sizeof(entry_service_desc)-1]='\x0';
+			if(temp_buffer)
+				entry_service_desc=strdup(temp_buffer);
+			else
+				entry_service_desc=NULL;
 
 			strncpy(image,SCHEDULED_DOWNTIME_ICON,sizeof(image));
 
@@ -658,8 +670,10 @@ void get_history(void){
 			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
-			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
-			entry_host_name[sizeof(entry_host_name)-1]='\x0';
+			if(temp_buffer)
+				entry_host_name=strdup(temp_buffer+1);
+			else
+				entry_host_name=NULL;
 
 			if(strstr(input,";DOWN;")){
 				strncpy(image,HOST_DOWN_ICON,sizeof(image));
@@ -690,8 +704,10 @@ void get_history(void){
 			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
-			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
-			entry_host_name[sizeof(entry_host_name)-1]='\x0';
+			if(temp_buffer)
+				entry_host_name=strdup(temp_buffer+1);
+			else
+				entry_host_name=NULL;
 
 			strncpy(image,FLAPPING_ICON,sizeof(image));
 
@@ -715,8 +731,10 @@ void get_history(void){
 			temp_buffer=my_strtok(input_buffer2,"]");
 			temp_buffer=my_strtok(NULL,":");
 			temp_buffer=my_strtok(NULL,";");
-			strncpy(entry_host_name,(temp_buffer==NULL)?"":temp_buffer+1,sizeof(entry_host_name));
-			entry_host_name[sizeof(entry_host_name)-1]='\x0';
+			if(temp_buffer)
+				entry_host_name=strdup(temp_buffer+1);
+			else
+				entry_host_name=NULL;
 
 			strncpy(image,SCHEDULED_DOWNTIME_ICON,sizeof(image));
 
@@ -901,11 +919,22 @@ void get_history(void){
 
 				if(display_frills==TRUE)
 					printf("<img align='left' src='%s%s' alt='%s' title='%s'>",url_images_path,image,image_alt,image_alt);
-				printf("[%s] %s<br clear='all'>\n",date_time,strip_plugin_html(temp_buffer));
+				printf("[%s] %s",date_time,html_encode(temp_buffer,FALSE));
+				if(enable_splunk_integration==TRUE){
+					printf("&nbsp;&nbsp;&nbsp;");
+					display_splunk_generic_url(temp_buffer,2);
+					}
+				printf("<br clear='all'>\n");
+
 				found_line=TRUE;
 			        }
 		        }
 
+		/* free memory */
+		free(entry_host_name);
+		entry_host_name=NULL;
+		free(entry_service_desc);
+		entry_service_desc=NULL;
                 }
 
 	printf("</P>\n");
