@@ -2,8 +2,8 @@
  *
  * TRENDS.C -  Nagios State Trends CGI
  *
- * Copyright (c) 1999-2007 Ethan Galstad (nagios@nagios.org)
- * Last Modified: 10-21-2007
+ * Copyright (c) 1999-2008 Ethan Galstad (nagios@nagios.org)
+ * Last Modified: 01-08-2008
  *
  * License:
  * 
@@ -39,6 +39,7 @@
 
 
 extern char main_config_file[MAX_FILENAME_LENGTH];
+extern char url_html_path[MAX_FILENAME_LENGTH];
 extern char url_images_path[MAX_FILENAME_LENGTH];
 extern char url_stylesheets_path[MAX_FILENAME_LENGTH];
 extern char physical_images_path[MAX_FILENAME_LENGTH];
@@ -1117,6 +1118,7 @@ void document_header(int use_stylesheet){
 
 		printf("<html>\n");
 		printf("<head>\n");
+		printf("<link rel=\"shortcut icon\" href=\"%sfavicon.ico\" type=\"image/ico\">\n",url_images_path);
 		printf("<title>\n");
 		printf("Nagios Trends\n");
 		printf("</title>\n");
@@ -1202,11 +1204,8 @@ int process_cgivars(void){
 				break;
 			        }
 
-			host_name=(char *)malloc(strlen(variables[x])+1);
-			if(host_name==NULL)
+			if((host_name=(char *)strdup(variables[x]))==NULL)
 				host_name="";
-			else
-				strcpy(host_name,variables[x]);
 			strip_html_brackets(host_name);
 
 			display_type=DISPLAY_HOST_TRENDS;
@@ -1220,11 +1219,8 @@ int process_cgivars(void){
 				break;
 			        }
 
-			svc_description=(char *)malloc(strlen(variables[x])+1);
-			if(svc_description==NULL)
+			if((svc_description=(char *)strdup(variables[x]))==NULL)
 				svc_description="";
-			else
-				strcpy(svc_description,variables[x]);
 			strip_html_brackets(svc_description);
 
 			display_type=DISPLAY_SERVICE_TRENDS;
@@ -2409,11 +2405,11 @@ void scan_log_file_for_archived_state_data(char *filename){
 	char *input2=NULL;
 	char entry_host_name[MAX_INPUT_BUFFER];
 	char entry_svc_description[MAX_INPUT_BUFFER];
-	char *plugin_output;
-	char *temp_buffer;
+	char *plugin_output=NULL;
+	char *temp_buffer=NULL;
 	time_t time_stamp;
-	mmapfile *thefile;
-	int state_type;
+	mmapfile *thefile=NULL;
+	int state_type=0;
 
 	/* print something so browser doesn't time out */
 	if(mode==CREATE_HTML){
