@@ -1,6 +1,6 @@
  package Embed::Persistent;
 
-# p1.pl for Nagios 2.0
+# p1.pl for Nagios
 
 use strict ;
 
@@ -88,10 +88,14 @@ sub PRINTF {
 
 sub READLINE {
 	my $self = shift;
+
+# CHANGED 12/26/07 EG Following two statements didn't allow for multi-line output or output > 256 chars
 								# Omit all lines after the first, per the nagios plugin guidelines
-        $$self = (split /\n/, $$self)[0];
+#        $$self = (split /\n/, $$self)[0];
 								# Perl code other than plugins may print nothing; in this case return "(No output!)\n".
-	return $$self ? substr($$self, 0, 256) : "(No output!)\n" ;
+#	return $$self ? substr($$self, 0, 256) : "(No output!)\n" ;
+
+	return $$self;
 }
 
 sub CLOSE {
@@ -201,6 +205,13 @@ EOSUB
 								# Suppress warning display.
 	local $SIG{__WARN__}			= \&throw_exception ;
 
+
+                                                                # Following 3 lines added 10/18/07 by Larry Low to fix problem where
+                                                                # modified Perl plugins didn't get recached by the epn
+        no strict 'refs';
+        undef %{$package.'::'};
+        use strict 'refs';
+
 								# Compile &$package::hndlr. Since non executable code is being eval'd
 								# there is no need to protect lexicals in this scope.
 	eval $hndlr;
